/*
******************************************************************************

   LibOVL: extension internals

   Copyright (C) 2001 Christoph Egger   [Christoph_Egger@t-online.de]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#ifndef _GGI_OVL_INT_H
#define _GGI_OVL_INT_H

#include <ggi/internal/internal.h>

#define HAVE_STRUCT_OVL
#include <ggi/ovl.h>

__BEGIN_DECLS

/* Exported variables */
#ifdef BUILDING_LIBOVL
extern ggi_extid    _ovlID;
#else
IMPORTVAR ggi_extid _ovlID;
#endif

__END_DECLS



/* Internal data types */

struct Common {
	/* pointer to a directbuffer.
	 * Note, that this may be shared with other sprites.
	 * If direct access to the sprite has to be emulated,
	 * these functions do _not_ have to use the read and
	 * write members of this db-struct.
	 */
	ggi_directbuffer *db;

	/* Pointers to the VRAM adress, if the sprite data
	 * can be accessed directly. Otherwise, they point
	 * to some RAM allocated from system memory.
	 */
	void *read;
	void *write;

	/* Pointer to set/get functions.  These functions will
	 * transfer the contents of a system RAM buffer to/from the
	 * sprite.  If you are writing to VRAM directly, the function
	 * will do nothing, but returning something like
	 * -E(INFO_DOESNOTHING).
	 */
	int (*set)(int x, int y, const void *data, int size);
	int (*get)(int x, int y, void *data, int size);
};


struct Sprite {

};

struct Video {

};

struct Window {

};


struct ggiOvl {
	enum OverlayFeaturetype ftype;
	ggi_visual_t vis;
	int x,y,w,h;
	int hidecount;

	struct Common common;
	union {
		struct Sprite sprite;
		struct Video  video;
		struct Window window;
	} Ovltype;
};



/* Data types for the API functions */
typedef ggiOvl_t (OvlAllocate)(ggi_visual_t vis, enum OverlayFeaturetype type,
			void *properties, int width, int height);

typedef int (OvlCheck)(ggi_visual_t vis, enum OverlayFeaturetype type,
                void *properties, int width, int height);

typedef int (OvlFree)(ggiOvl_t ovl);

typedef int (OvlSetImage)(ggiOvl_t ovl, ggi_visual_t vis,
			int x, int y, unsigned char *alpha);
typedef int (OvlSetOption)(ggiOvl_t ovl,
			enum OverlayOptiontype op, void *arg);

typedef int (OvlMove)(ggiOvl_t ovl, int x, int y);	

typedef int (OvlShow)(ggiOvl_t ovl);
typedef int (OvlHide)(ggiOvl_t ovl);
typedef int (OvlIsHidden)(ggiOvl_t ovl);


typedef struct ovlpriv {
	OvlAllocate	*alloc;
	OvlCheck	*check;
	OvlFree		*free;

	OvlSetImage	*setimage;
	OvlSetOption	*setoption;

	OvlMove		*move;

	OvlShow		*show;
	OvlHide		*hide;
	OvlIsHidden	*ishidden;
	
	

	void *priv;	/* Here sublibs can store visual specific private data.
			   If you have more than one sublib (for each visual)
			   that needs private data, just add more pointers or
			   add the data fields directly in this structure. */
} ovlpriv;

/* Used to access the ovlpriv structure for a particular visual. */
#define LIBGGI_OVLEXT(vis) ((ovlpriv *)LIBGGI_EXT((vis),_ovlID))


#endif /* _GGI_OVL_INT_H */
